/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can read the License at http://opensource.org/licenses/CDDL-1.0
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL Header Notice in
 * each file and include the License file (licence.txt). If applicable,
 * add the following below this CDDL Header, with the fields enclosed
 * by brackets [] replaced by your own identifying information:
 * "Portions Copyright [year] [name of copyright owner]"
 *
 * The Original Software is AtmoSwing.
 * The Original Software was developed at the University of Lausanne.
 * All Rights Reserved.
 *
 */

/*
 * Portions Copyright 2008-2013 Pascal Horton, University of Lausanne.
 * Portions Copyright 2013-2015 Pascal Horton, Terranum.
 */

#include "asResultsTotalScore.h"

#include "asFileNetcdf.h"
#include "asParametersScoring.h"

asResultsTotalScore::asResultsTotalScore() : asResults(), m_hasSingleValue(true), m_score(NaNf) {}

asResultsTotalScore::~asResultsTotalScore() {}

void asResultsTotalScore::Init() {
  // Set to nan to avoid keeping old results
  m_score = NaNf;
  m_scoreArray.resize(0);
}

void asResultsTotalScore::BuildFileName() {
  ThreadsManager().CritSectionConfig().Enter();
  m_filePath = wxFileConfig::Get()->Read("/Paths/ResultsDir", asConfig::GetDefaultUserWorkingDir());
  ThreadsManager().CritSectionConfig().Leave();
  if (!m_subFolder.IsEmpty()) {
    m_filePath.Append(DS);
    m_filePath.Append(m_subFolder);
  }
  m_filePath.Append(DS);
  m_filePath.Append(wxString::Format("TotalScore_id_%s_step_%d", GetPredictandStationIdsList(), m_currentStep));
  m_filePath.Append(".nc");
}

bool asResultsTotalScore::Save() {
  BuildFileName();

  ThreadsManager().CritSectionNetCDF().Enter();

  // Create netCDF dataset: enter define mode
  asFileNetcdf ncFile(m_filePath, asFileNetcdf::Replace);
  if (!ncFile.Open()) {
    ThreadsManager().CritSectionNetCDF().Leave();
    return false;
  }

  // Define dimensions.
  ncFile.DefDim("score");

  // The dimensions name array is used to pass the dimensions to the variable.
  vstds dimNames1;
  dimNames1.push_back("score");

  // Define variables
  ncFile.DefVar("score", NC_FLOAT, 1, dimNames1);

  // Put attributes
  DefTotalScoreAttributes(ncFile);

  // End definitions: leave define mode
  ncFile.EndDef();

  // Provide sizes for variables
  size_t start1D[] = {0};
  size_t count1D[] = {1};

  // Write data
  ncFile.PutVarArray("score", start1D, count1D, &m_score);

  // Close:save new netCDF dataset
  ncFile.Close();

  ThreadsManager().CritSectionNetCDF().Leave();

  return true;
}

bool asResultsTotalScore::Load() {
  // Makes no sense to load at this stage.
  return false;
}
